@@ -22,6 +22,7 @@
 #define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
 #define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
 #define g_marshal_value_peek_object(v)   g_value_get_object (v)
+#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
 #else /* !G_ENABLE_DEBUG */
 /* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
  *          Do not access GValues directly in your code. Instead, use the
@@ -45,16 +46,17 @@
 #define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
 #define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
 #define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
+#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
 
 /* BOOLEAN:VOID (./genmarshal:1) */
 void
 gnt_closure_marshal_BOOLEAN__VOID (GClosure     *closure,
-                                   GValue       *return_value,
+                                   GValue       *return_value G_GNUC_UNUSED,
                                    guint         n_param_values,
                                    const GValue *param_values,
-                                   gpointer      invocation_hint,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                    gpointer      marshal_data)
 {
   typedef gboolean (*GMarshalFunc_BOOLEAN__VOID) (gpointer     data1,
@@ -88,10 +90,10 @@ gnt_closure_marshal_BOOLEAN__VOID (GClosure     *closure,
 /* BOOLEAN:STRING (./genmarshal:2) */
 void
 gnt_closure_marshal_BOOLEAN__STRING (GClosure     *closure,
-                                     GValue       *return_value,
+                                     GValue       *return_value G_GNUC_UNUSED,
                                      guint         n_param_values,
                                      const GValue *param_values,
-                                     gpointer      invocation_hint,
+                                     gpointer      invocation_hint G_GNUC_UNUSED,
                                      gpointer      marshal_data)
 {
   typedef gboolean (*GMarshalFunc_BOOLEAN__STRING) (gpointer     data1,
@@ -127,10 +129,10 @@ gnt_closure_marshal_BOOLEAN__STRING (GClosure     *closure,
 /* VOID:INT,INT,INT,INT (./genmarshal:3) */
 void
 gnt_closure_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
-                                           GValue       *return_value,
+                                           GValue       *return_value G_GNUC_UNUSED,
                                            guint         n_param_values,
                                            const GValue *param_values,
-                                           gpointer      invocation_hint,
+                                           gpointer      invocation_hint G_GNUC_UNUSED,
                                            gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_INT_INT_INT) (gpointer     data1,
@@ -168,10 +170,10 @@ gnt_closure_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
 /* VOID:INT,INT (./genmarshal:4) */
 void
 gnt_closure_marshal_VOID__INT_INT (GClosure     *closure,
-                                   GValue       *return_value,
+                                   GValue       *return_value G_GNUC_UNUSED,
                                    guint         n_param_values,
                                    const GValue *param_values,
-                                   gpointer      invocation_hint,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                    gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_INT) (gpointer     data1,
@@ -205,10 +207,10 @@ gnt_closure_marshal_VOID__INT_INT (GClosure     *closure,
 /* VOID:POINTER,POINTER (./genmarshal:5) */
 void
 gnt_closure_marshal_VOID__POINTER_POINTER (GClosure     *closure,
-                                           GValue       *return_value,
+                                           GValue       *return_value G_GNUC_UNUSED,
                                            guint         n_param_values,
                                            const GValue *param_values,
-                                           gpointer      invocation_hint,
+                                           gpointer      invocation_hint G_GNUC_UNUSED,
                                            gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__POINTER_POINTER) (gpointer     data1,
@@ -242,10 +244,10 @@ gnt_closure_marshal_VOID__POINTER_POINTER (GClosure     *closure,
 /* BOOLEAN:INT,INT (./genmarshal:6) */
 void
 gnt_closure_marshal_BOOLEAN__INT_INT (GClosure     *closure,
-                                      GValue       *return_value,
+                                      GValue       *return_value G_GNUC_UNUSED,
                                       guint         n_param_values,
                                       const GValue *param_values,
-                                      gpointer      invocation_hint,
+                                      gpointer      invocation_hint G_GNUC_UNUSED,
                                       gpointer      marshal_data)
 {
   typedef gboolean (*GMarshalFunc_BOOLEAN__INT_INT) (gpointer     data1,
@@ -283,10 +285,10 @@ gnt_closure_marshal_BOOLEAN__INT_INT (GClosure     *closure,
 /* BOOLEAN:INT,INT,INT (./genmarshal:7) */
 void
 gnt_closure_marshal_BOOLEAN__INT_INT_INT (GClosure     *closure,
-                                          GValue       *return_value,
+                                          GValue       *return_value G_GNUC_UNUSED,
                                           guint         n_param_values,
                                           const GValue *param_values,
-                                          gpointer      invocation_hint,
+                                          gpointer      invocation_hint G_GNUC_UNUSED,
                                           gpointer      marshal_data)
 {
   typedef gboolean (*GMarshalFunc_BOOLEAN__INT_INT_INT) (gpointer     data1,
@@ -326,10 +328,10 @@ gnt_closure_marshal_BOOLEAN__INT_INT_INT (GClosure     *closure,
 /* BOOLEAN:POINTER,POINTER,POINTER (./genmarshal:8) */
 void
 gnt_closure_marshal_BOOLEAN__POINTER_POINTER_POINTER (GClosure     *closure,
-                                                      GValue       *return_value,
+                                                      GValue       *return_value G_GNUC_UNUSED,
                                                       guint         n_param_values,
                                                       const GValue *param_values,
-                                                      gpointer      invocation_hint,
+                                                      gpointer      invocation_hint G_GNUC_UNUSED,
                                                       gpointer      marshal_data)
 {
   typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER_POINTER_POINTER) (gpointer     data1,
@@ -369,10 +371,10 @@ gnt_closure_marshal_BOOLEAN__POINTER_POINTER_POINTER (GClosure     *closure,
 /* BOOLEAN:INT,INT,INT,POINTER (./genmarshal:9) */
 void
 gnt_closure_marshal_BOOLEAN__INT_INT_INT_POINTER (GClosure     *closure,
-                                                  GValue       *return_value,
+                                                  GValue       *return_value G_GNUC_UNUSED,
                                                   guint         n_param_values,
                                                   const GValue *param_values,
-                                                  gpointer      invocation_hint,
+                                                  gpointer      invocation_hint G_GNUC_UNUSED,
                                                   gpointer      marshal_data)
 {
   typedef gboolean (*GMarshalFunc_BOOLEAN__INT_INT_INT_POINTER) (gpointer     data1,
@@ -414,10 +416,10 @@ gnt_closure_marshal_BOOLEAN__INT_INT_INT_POINTER (GClosure     *closure,
 /* VOID:STRING,STRING (./genmarshal:10) */
 void
 gnt_closure_marshal_VOID__STRING_STRING (GClosure     *closure,
-                                         GValue       *return_value,
+                                         GValue       *return_value G_GNUC_UNUSED,
                                          guint         n_param_values,
                                          const GValue *param_values,
-                                         gpointer      invocation_hint,
+                                         gpointer      invocation_hint G_GNUC_UNUSED,
                                          gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer     data1,
@@ -451,10 +453,10 @@ gnt_closure_marshal_VOID__STRING_STRING (GClosure     *closure,
 /* VOID:POINTER,BOOLEAN (./genmarshal:11) */
 void
 gnt_closure_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
-                                           GValue       *return_value,
+                                           GValue       *return_value G_GNUC_UNUSED,
                                            guint         n_param_values,
                                            const GValue *param_values,
-                                           gpointer      invocation_hint,
+                                           gpointer      invocation_hint G_GNUC_UNUSED,
                                            gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__POINTER_BOOLEAN) (gpointer     data1,
